from flask import Flask, request, jsonify
import os
import dotenv
import pgos_lib
import conf

# Load environment variables
dotenv.load_dotenv()

app = Flask(__name__)

@app.route("/", methods=["GET"])
def hello():
    return "Hello World!"

# client_hello invoke respond a simple 'Hello' to the invoker, ONLY for client side invoking
@app.route("/client_hello", methods=["POST"])
def client_hello():
    if request.headers.get("Invoker") != "client":
        return "", 401
    name = request.data.decode("utf-8")
    return f"Hello client {name}!"

# backend_hello invoke respond a simple 'Hello' to the invoker, ONLY for backend side invoking(by backend http api)
@app.route("/backend_hello", methods=["POST"])
def backend_hello():
    if request.headers.get("Invoker") != "backend":
        return "", 401
    name = request.data.decode("utf-8")
    return f"Hello backend {name}!"

# backend_get_player_info is a sample uri to demostrate backend side invoking
# it invokes backend http api to get a player's info, and respond to the invoker, ONLY for backend side invoking(by backend http api)
# you can invoke any http api based on your demand, even call other virtual server :-)
@app.route("/backend_get_player_info", methods=["POST"])
def backend_get_player_info():
    if request.headers.get("Invoker") != "backend":
        return "", 401
    headers, body = pgos_lib.invoke_backend_api(request.headers.get("Titleid"), request.headers.get("Titleregionid"), conf.ServerKey, "/player_svr/get_player_info", request.get_data())
    return body

# pgos_event handles all event, please config the event handler in the portal in advance
@app.route("/pgos_event", methods=["POST"])
def pgos_event():
    if request.headers.get("Invoker") != "event":
        return "", 401
    # Write your own code to handle the event
    print("/pgos_event: ", request.data.decode("utf-8"))
    return "", 200

# pgos_economy handles the specific requests from PGOS economy service, please config the binding for the economy service on the portal in advance
@app.route("/pgos_economy", methods=["POST"])
def pgos_economy():
    if request.headers.get("Invoker") != "economy":
        return "", 401
    # Write your own code to handle
    print("/pgos_economy: ", request.data.decode("utf-8"))
    return "", 200

# pgos_task handles the task request, please config the task on the portal in advance
@app.route("/pgos_task", methods=["POST"])
def pgos_task():
    if request.headers.get("Invoker") != "task":
        return "", 401
    # Write your own code to handle
    print("/pgos_task: ", request.data.decode("utf-8"))
    return "", 200

if __name__ == "__main__":
    port = os.getenv("PORT") or "8081"
    app.run(host="0.0.0.0", port=int(port))