# a lib to help you invoking PGOS backend Http API
from Crypto.Cipher import AES
import base64
import requests
import json

ServerTokenIv = b"$3,.'/&^rgnjkl!#"

class AesCbc:
    def __init__(self, key, iv):
        self.key = key
        self.iv = iv

    def encrypt(self, text):
        cipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        padded_text = self.pad(text.encode('utf-8'))
        encrypted = cipher.encrypt(padded_text)
        return base64.b64encode(encrypted).decode('utf-8')

    def decrypt(self, text):
        decipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        decoded_text = base64.b64decode(text)
        decrypted = decipher.decrypt(decoded_text)
        return self.unpad(decrypted).decode('utf-8')

    def pad(self, text):
        block_size = 16
        padding = block_size - (len(text) % block_size)
        pad_text = bytes([padding]) * padding
        return text + pad_text

    def unpad(self, text):
        unpadding = text[-1]
        return text[:len(text) - unpadding]


def encrypt_server_ticket(key, ticket_json_str):
    ac = AesCbc(key.encode('utf-8'), ServerTokenIv)
    return ac.encrypt(ticket_json_str)


def decrypt_server_ticket(key, ticket_json_str):
    ac = AesCbc(key.encode('utf-8'), ServerTokenIv)
    return ac.decrypt(ticket_json_str)


class SecretTicket:
    def __init__(self, title_region_id, secret_id, t):
        self.title_region_id = title_region_id
        self.secret_id = secret_id
        self.time = t


def gen_server_ticket(secret_id, secret_key, title_region_id):
    import time
    
    st = SecretTicket(title_region_id, secret_id, int(time.time()))
    ticket_json = json.dumps(st.__dict__)
    key = secret_key.replace("-", "")
    ticket = encrypt_server_ticket(key, ticket_json)
    return ticket


def split_server_key(server_key):
    s = server_key.split("-")
    if len(s) >= 2:
        secret_id = s[0]
        secret_key = '-'.join(s[1:])
        return secret_id, secret_key
    else:
        return None, None


def get_server_url(title_region_id):
    parts = title_region_id.split('_')
    location = parts[0] if len(parts) > 0 else 'euff'
    domain_suffix = 'com'

    if location == 't' or location == 'd' or location.startswith('cn'):
        domain_suffix = 'cn'

    server_url = f'https://{location}.server.pgos.intlgame.{domain_suffix}'
    return server_url


def invoke_backend_api(title_id, title_region_id, server_key, api_uri, body_str):
    secret_id, secret_key = split_server_key(server_key)
    if secret_id is None or secret_key is None:
        return None

    ticket = gen_server_ticket(secret_id, secret_key, title_region_id)
    url = get_server_url(title_region_id) + api_uri
    headers = {
        "Content-Type": "application/json",
        "TitleId": title_id,
        "TitleRegionId": title_region_id,
        "SecretId": secret_id,
        "ServerTicket": ticket
    }

    response = requests.post(url, body_str, headers=headers, timeout=3)
    return response.headers, response.text